using System;
using System.Collections.Generic;
using System.ComponentModel;
using Gecko.Interfaces;
using Gecko.Interop;
using Gecko.Windows;

namespace Gecko.Control
{
    public class WebViewAjax : WebView, nsIHttpActivityObserver
    {
        public class AjaxChannel : nsIDOMEventListener
        {
            private readonly WebViewAjax _webViewAjax;
            private readonly nsIHttpChannel _nsHttpChannel;
            private readonly ComObject<nsIXMLHttpRequest> _notificationCallsbacks;

            public AjaxChannel(WebViewAjax webViewAjax, nsIHttpChannel nsHttpChannel)
            {
                _webViewAjax = webViewAjax;
                _nsHttpChannel = nsHttpChannel;
                _notificationCallsbacks = Xpcom.QueryInterface<nsIXMLHttpRequest>(_nsHttpChannel.GetNotificationCallbacksAttribute()).AsComObject();
            }

            public void HandleEvent(nsIDOMEvent domEvent)
            {
                nsIXMLHttpRequest ci = _notificationCallsbacks.Instance;
                var xhrReadyState = ci.GetReadyStateAttribute();
                if (xhrReadyState == 4)
                {
                    _webViewAjax.RemoveAjaxChannel(_nsHttpChannel);
                }
            }
        }
        
        private readonly Dictionary<nsIHttpChannel, AjaxChannel> _activeAjaxChannels = new Dictionary<nsIHttpChannel, AjaxChannel>();

        [Browsable(false)]
        public bool IsBusyAjax { get; private set; }

        public void RemoveAjaxChannel(nsIHttpChannel channel)
        {
            _activeAjaxChannels.Remove(channel);
            IsBusyAjax = _activeAjaxChannels.Count > 0;
        }

        protected override void OnHandleCreated(EventArgs e)
        {
            base.OnHandleCreated(e);
            if (!this.DesignMode)
            {
                using (var activityDistributor = Xpcom.GetService2<nsIHttpActivityDistributor>(Contracts.HttpActivityDistributor))
                {
                    activityDistributor.Instance.AddObserver(this);
                }
            }
        }

        public void ObserveActivity(nsISupports aHttpChannel, uint aActivityType, uint aActivitySubtype, ulong aTimestamp, ulong aExtraSizeData, nsACStringBase aExtraStringData)
        {
            switch ((ulong)aActivityType)
            {

                case nsIHttpActivityObserverConsts.ACTIVITY_TYPE_SOCKET_TRANSPORT:
                    using (var channel = Xpcom.QueryInterface2<nsIChannel>(aHttpChannel))
                    {
                        if (channel == null)
                            break;
                        switch ((ulong)aActivitySubtype)
                        {
                            case nsISocketTransportConsts.STATUS_RESOLVING:
                                break;
                            case nsISocketTransportConsts.STATUS_RESOLVED:
                                break;
                            case nsISocketTransportConsts.STATUS_CONNECTING_TO:
                                break;
                            case nsISocketTransportConsts.STATUS_CONNECTED_TO:
                                break;
                            case nsISocketTransportConsts.STATUS_SENDING_TO:
                                break;
                            case nsISocketTransportConsts.STATUS_WAITING_FOR:
                                break;
                            case nsISocketTransportConsts.STATUS_RECEIVING_FROM:
                                break;
                            default:
                                break;
                        }
                    }
                    break;
                case nsIHttpActivityObserverConsts.ACTIVITY_TYPE_HTTP_TRANSACTION:
                    using (var httpChannel = Xpcom.QueryInterface2<nsIHttpChannel>(aHttpChannel))
                    {
                        if (httpChannel == null)
                            break;

                        switch ((ulong)aActivitySubtype)
                        {
                            case nsIHttpActivityObserverConsts.ACTIVITY_SUBTYPE_REQUEST_HEADER:
                            {
                                try
                                {
                                    using (var callbacks = httpChannel.Instance.GetNotificationCallbacksAttribute().AsComObject())
                                    {
                                        if (callbacks == null)
                                            break;

                                        using (var httpChannelXhr = callbacks.QueryInterface<nsIXMLHttpRequest>().AsComObject())
                                        {
                                            if (httpChannelXhr == null)
                                                break;

                                            using (var mXmlRequestEvent = httpChannelXhr.QueryInterface<nsIXMLHttpRequestEventTarget>().AsComObject())
                                            {
                                                if (mXmlRequestEvent == null)
                                                    break;

                                                var ajaxChannel = new AjaxChannel(this, httpChannel.Instance);
                                                using (var mLoads = new nsAString("load"))
                                                    mXmlRequestEvent.Instance.AddEventListener(mLoads, ajaxChannel, true, false, 0);

                                                using (var mLoads = new nsAString("abort"))
                                                    mXmlRequestEvent.Instance.AddEventListener(mLoads, ajaxChannel, true, false, 0);

                                                using (var mLoads = new nsAString("error"))
                                                    mXmlRequestEvent.Instance.AddEventListener(mLoads, ajaxChannel, true, false, 0);
                                                _activeAjaxChannels.Add(httpChannel.Instance, ajaxChannel);
                                                IsBusyAjax = true;
                                            }
                                        }
                                    }
                                }
                                catch (Exception ex)
                                {
                                }
                            }
                                break;
                            case nsIHttpActivityObserverConsts.ACTIVITY_SUBTYPE_RESPONSE_HEADER:
                                break;
                            case nsIHttpActivityObserverConsts.ACTIVITY_SUBTYPE_REQUEST_BODY_SENT:
                                break;
                            case nsIHttpActivityObserverConsts.ACTIVITY_SUBTYPE_RESPONSE_START:
                                break;
                            case nsIHttpActivityObserverConsts.ACTIVITY_SUBTYPE_RESPONSE_COMPLETE:
                                break;
                            case nsIHttpActivityObserverConsts.ACTIVITY_SUBTYPE_TRANSACTION_CLOSE:
                                break;
                            default:
                                break;
                        }
                    }
                    break;
                default:
                    break;
            }
        }

        public bool GetIsActiveAttribute()
        {
            return true;
        }

    }
}